﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Sample
{
    public partial class MainForm : Form
    {
        // デコード範囲指定（formSetRectで設定）
        public static bool enableSetRect = false;
        public static int rectLeft = 0;
        public static int rectTop = 0;
        public static int rectRight = 0;
        public static int rectBottom = 0;

        // デコード条件設定（formSetPropertyで設定）
        public static int symbolCount = 0;
        public static int effectLevel = 3;
        public static byte border = 0;
        public static byte moduleSize = 1;

        private int filterIndex = 1;

        public MainForm()
        {
            InitializeComponent();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            // テキストボックスをクライアント領域に合わせて表示
            txtDecodeResult.Width = ClientRectangle.Width;
            txtDecodeResult.Height = ClientRectangle.Height - menuStrip1.Height;
        }

        private void txtDecodeResult_DragEnter(object sender, DragEventArgs e)
        {
            // ファイルのみドラッグ＆ドロップを許可
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Copy;
        }

        private void txtDecodeResult_DragDrop(object sender, DragEventArgs e)
        {
            // ファイルドロップ時処理
            DecodeFile(((string[])e.Data.GetData(DataFormats.FileDrop))[0]);
        }

        private void menuOpenFile_Click(object sender, EventArgs e)
        {
            // [ファイル]-[開く]処理

            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.Filter = "ピクチャ ファイル (*.bmp;*.jpg;*.jpeg;*.gif;*.tif;*.tiff;*.png)|*.bmp;*.jpg;*.jpeg;*.gif;*.tif;*.tiff;*.png|" +
                                     "ビットマップ ファイル (*.bmp)|*.bmp|" +
                                     "JPEG 形式 (*.jpg;*.jpeg)|*.jpg;*.jpeg|" +
                                     "GIF 形式 (*.gif)|*.gif|" +
                                     "TIFF 形式 (*.tif;*.tiff)|*.tif;*.tiff|" +
                                     "PNG 形式 (*.png)|*.png|" +
                                     "すべてのファイル (*.*)|*.*";
            openFileDialog1.FilterIndex = filterIndex;
            openFileDialog1.RestoreDirectory = true;

            // コモンダイアログによりファイルを選択
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                filterIndex = openFileDialog1.FilterIndex;
                
                DecodeFile(openFileDialog1.FileName);
            }
        }

        private void menuAppExit_Click(object sender, EventArgs e)
        {
            //プログラム終了
            Close();
        }

        private void menuSetDecodeRect_Click(object sender, EventArgs e)
        {
            // デコード範囲指定ダイアログ表示
            SetRectForm setRectForm = new SetRectForm();

            setRectForm.ShowDialog(); // モーダル表示

            setRectForm.Dispose();
        }

        private void menuSetDecodeProperty_Click(object sender, EventArgs e)
        {
            // デコード条件設定ダイアログ表示
            SetPropertyForm setPropertyForm = new SetPropertyForm();

            setPropertyForm.ShowDialog(); // モーダル表示

            setPropertyForm.Dispose();
        }

        private void DecodeFile(String strFilePath)
        {
            // 画像ファイルを対象としたデコード処理

            string strSeparator = "───────────────────────────\r\n";
            string[] strLevel = { "L(7%)", "M(15%)", "Q(25%)", "H(30%)" };

            // 砂時計カーソルを表示
            Cursor = System.Windows.Forms.Cursors.WaitCursor;

            // DLLファンクションによるデコード
            int nResult;

            if (enableSetRect)
            {
                // 範囲指定あり
                nResult = PsyQrDcd.DecodePictureFileRect(strFilePath, rectLeft, rectTop, rectRight, rectBottom);
            }
            else
            {    // 範囲指定なし
                nResult = PsyQrDcd.DecodePictureFile(strFilePath);
            }

            // カーソル復元
            Cursor = System.Windows.Forms.Cursors.Default;

            string strResult = ""; // デコード結果文字列（テキストボックス表示用）

            if (nResult >= 1)
            {
                // デコード正常終了
                for (int i = 0; i < nResult; ++i)
                {
                    strResult += strSeparator;

                    string strWork;

                    // シンボル型番、誤り訂正レベル
                    strWork = string.Format("デコード データ {0} [型番:{1}, レベル:{2}", i + 1, PsyQrDcd.GetDecodeVersion(i), strLevel[PsyQrDcd.GetDecodeLevel(i)]);
                    strResult += strWork;

                    // シンボル連結情報
                    int nSeqNo = 0;
                    byte byCheckDigit = 0;

                    int ncConcatenation = PsyQrDcd.GetConcatenationInfo(i, ref nSeqNo, ref byCheckDigit);

                    if (ncConcatenation > 0)
                    {
                        strWork = string.Format(", 連結:{0}/{1}(CD:{2:X2}H)", nSeqNo + 1, ncConcatenation, byCheckDigit);
                        strResult += strWork;
                    }

                    strResult += "]\r\n";

                    strResult += strSeparator;

                    // シンボルデータ
                    StringBuilder buffer = new StringBuilder(7089);

                    PsyQrDcd.GetDecodeDataString(i, ref buffer, menuPrioritizeUTF8.Checked);
                    
                    strResult += buffer;

                    strResult += "\r\n\r\n";

                    // シンボル位置情報
                    int[] nPosition = new int[8];

                    PsyQrDcd.GetSymbolePosition(i, nPosition);

                    strWork = string.Format("シンボル位置：({0}, {1})-({2}, {3})-({4}, {5})-({6}, {7})",
                                            nPosition[0], nPosition[1], nPosition[2], nPosition[3],
                                            nPosition[4], nPosition[5], nPosition[6], nPosition[7]);
                    strResult += strWork;
                    strResult += "\r\n\r\n";
                }

                // デコード結果保存メモリを開放
                PsyQrDcd.FreeAllocateMemory();
            }
            else
            {
                switch (nResult)
                {
                    case (int)DecodeResult.QRD_ERROR_SYMBLE_NOT_FOUND:
                        strResult = "QRコード シンボルを読み取れませんでした。";

                        switch (PsyQrDcd.GetErrorLevel())
                        {
                            case 1:
                                strResult += "\r\n";
                                strResult += "(位置検出パターンが見つかりませんでした。)";
                                break;

                            case 2:
                                strResult += "\r\n";
                                strResult += "(位置検出パターンからシンボルを認識できませんでした。)";
                                break;

                            case 3:
                                strResult += "\r\n";
                                strResult += "(データ部の読み取りに失敗しました。)";
                                break;

                            default:
                                break;
                        }

                        break;

                    case (int)DecodeResult.QRD_ERROR_FILE_NOT_FOUND:
                        strResult = "ファイルが見つかりませんでした。";
                        break;

                    case (int)DecodeResult.QRD_ERROR_READ_FAULT:
                        strResult = "ファイル読み込み中にエラーが発生しました。";
                        break;

                    case (int)DecodeResult.QRD_ERROR_BAD_FORMAT:
                        strResult = "読み取りできないファイル種別です。";
                        break;

                    case (int)DecodeResult.QRD_ERROR_SHARING_VIOLATION:
                        strResult = "共有違反によりファイルを読み取ることができませんでした。";
                        break;

                    case (int)DecodeResult.QRD_ERROR_NOT_ENOUGH_MEMORY:
                        strResult = "メモリ不足です。";
                        break;

                    default:
                        break;
                }
            }

            txtDecodeResult.Text = strResult;
        }
    }
}
